//+------------------------------------------------------------------+
//|                                                         TSIs.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 8
#property indicator_plots   2
//#property indicator_applied_price PRICE_TYPICAL
//---      MovingAverages.mqh
#include <MovingAverages.mqh>
//---- plot TSI
#property indicator_label1  "TSI"
#property indicator_type1   DRAW_LINE
#property indicator_color1  Blue
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//---- plot TSI Signal
#property indicator_label2 "TSISignal" //            .
#property indicator_type2 DRAW_LINE //   
#property indicator_color2 Red //  
#property indicator_style2 STYLE_SOLID // 
#property indicator_width2 1 //  
//--- input parameters
input int r=25;
input int s=13;
input int sp=5;
input ENUM_MA_METHOD sm=MODE_EMA;
//--- indicator buffers
double         TSIBuffer[];
double         TSISigBuffer[];
double         MTMBuffer[];
double         AbsMTMBuffer[];
double         EMA_MTMBuffer[];
double         EMA2_MTMBuffer[];
double         EMA_AbsMTMBuffer[];
double         EMA2_AbsMTMBuffer[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,TSIBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,TSISigBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,MTMBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,AbsMTMBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,EMA_MTMBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,EMA2_MTMBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(6,EMA_AbsMTMBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(7,EMA2_AbsMTMBuffer,INDICATOR_CALCULATIONS);
//---      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,r+s-1);
   string shortname;
   StringConcatenate(shortname,"TSI(",r,",",s,")");
//---      DataWindow
   PlotIndexSetString(0,PLOT_LABEL,shortname);
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,2);
//---
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,r+s+sp);
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate (const int rates_total,    //   price[];
                 const int prev_calculated,//    ;
                 //   ;
                 const int begin,//      
                 // price[]   ;
                 const double &price[]) // ,      ;
  {
//---      price[]
   static bool printed=false;
//---  begin   ,   ,       
   if(begin>0 && !printed)
     {
      //--- 
      Print("      begin =",begin,
            "     price[] =",rates_total);

      //---    ,     
      for(int i=0;i<=begin;i++)
        {
         Print("i =",i,"  value =",price[i]);
        }
      //---   ,      
      printed=true;
     }
//---    price[]  
   if(rates_total<r+s) return(0); //         
//---     
   if(prev_calculated==0)
     { 
      //---      begin ,      
      if(begin>0)PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,begin+r+s-1);
      //---     EMPTY_VALUE
      ArrayInitialize(TSIBuffer,EMPTY_VALUE);
      ArrayInitialize(MTMBuffer,EMPTY_VALUE);
      ArrayInitialize(AbsMTMBuffer,EMPTY_VALUE);
      ArrayInitialize(EMA_MTMBuffer,EMPTY_VALUE);
      ArrayInitialize(EMA2_MTMBuffer,EMPTY_VALUE);
      ArrayInitialize(EMA_AbsMTMBuffer,EMPTY_VALUE);
      ArrayInitialize(EMA2_AbsMTMBuffer,EMPTY_VALUE);
      //---      
      MTMBuffer[0]=0.0;
      AbsMTMBuffer[0]=0.0;
     }

//---   mtm  |mtm|
   int start;
   if(prev_calculated==0) start=begin+1;  //   MTMBuffer[]  AbsMTMBuffer[]    begin+1
   else start=prev_calculated-1;          //  start      
   for(int i=start;i<rates_total;i++)
     {
      MTMBuffer[i]=price[i]-price[i-1];
      AbsMTMBuffer[i]=fabs(MTMBuffer[i]);
     }

//---      
   ExponentialMAOnBuffer(rates_total,prev_calculated,
                         begin+1,//          
                         r,      //   
                         MTMBuffer,       //    
                         EMA_MTMBuffer);  //      
   ExponentialMAOnBuffer(rates_total,prev_calculated,
                         begin+1,r,AbsMTMBuffer,EMA_AbsMTMBuffer);

//---      
   ExponentialMAOnBuffer(rates_total,prev_calculated,
                         begin+r,s,EMA_MTMBuffer,EMA2_MTMBuffer);
   ExponentialMAOnBuffer(rates_total,prev_calculated,
                         begin+r,s,EMA_AbsMTMBuffer,EMA2_AbsMTMBuffer);

//---    
   if(prev_calculated==0) start=begin+r+s-1; //      
   else start=prev_calculated-1;             //  start      
   for(int i=start;i<rates_total;i++)
     {
      TSIBuffer[i]=100*EMA2_MTMBuffer[i]/EMA2_AbsMTMBuffer[i];
     }
   int begin2=begin+r+s-1;     
   switch(sm)
     {
      case MODE_EMA:
         ExponentialMAOnBuffer(rates_total,prev_calculated,begin2,sp,TSIBuffer,TSISigBuffer);
         break;
      case MODE_LWMA:
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,begin2,sp,TSIBuffer,TSISigBuffer);
         break;
      case MODE_SMA:
         SimpleMAOnBuffer(rates_total,prev_calculated,begin2,sp,TSIBuffer,TSISigBuffer);
         break;
      case MODE_SMMA:
         SmoothedMAOnBuffer(rates_total,prev_calculated,begin2,sp,TSIBuffer,TSISigBuffer);
         break;
     }   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
